<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img
      class="logo"
      src="{{ pathto('_static/greenbone.png', 1) }}"
      alt="Greenbone Logo"
    />
    <h1 class="logo logo-name">{{ project }}</h1>
  </a>
</p>
<p class="description">
  Remote Control of Your Greenbone Vulnerability Manager (GVM)
</p>
